@@ -29,10 +29,33 @@ import gnome.applet
 import gnome.ui
 import os.path
 import re
+import copy
 import gobject
 import pycentrino_globals as pglobals
 
-# the class inherits from gnome.applet.Applet and we'll create it as a gobject (see __init__())
+def delete_node(nodo):
+    if hasattr(nodo,get_children):
+        if nodo.get_children()!= []:
+            nodo.remove(nodo.get_children()[0])
+            delete_node(nodo)
+        else:
+            del nodo
+            return
+    else:
+        del nodo
+        return
+ 
+def delete_widget(widget):
+    parent = None
+    if hasattr(widget,'get_children'):
+        for child in widget.get_children():
+            print "borrando relacion " + str(widget) + str(child)
+            widget.remove(child)
+            delete_widget(child)
+            parent = widget.get_parent()
+        if parent != None:
+            parent.remove(widget)
+
 class pycentrinoApplet(gnome.applet.Applet):
         
     def cleanup(self,event):
@@ -40,45 +63,76 @@ class pycentrinoApplet(gnome.applet.Applet):
     
     # this callback is for update the applet from system info every 1 second
     def timeout_callback(self,event):
-        return 0
+        self.read_proc_info()
+        self.orientation = self.applet.get_orient()
+
+        for i in (self.box.get_children()):
+            self.box.remove(i)
+
+        cont = -1
+        separate = 0
+        # create the applet with little hboxes containing each info checked in the preferences
+        for i in range(len(self.checks)):
+            if self.checks[i]:
+                cont = cont + 1
+
+                self.box.pack_start(gtk.EventBox())
+                
+                # add un hbox para meter imagenes
+                self.box.get_children()[cont].add(gtk.HBox())
+                if separate:
+                    tmp = gtk.Image()
+                    tmp.set_from_file(os.path.join(pglobals.image_dir,'separator.gif'))
+                    self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+
+                
+                for h in range(len(self.info[i])):
+                    tmp = gtk.Image()
+                    tmp.set_from_file(os.path.join(pglobals.image_dir,'green_led_%s.gif' % self.info[i][h]))
+                    self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                separate = 1
+
+        #self.applet.hide()
+        self.applet.show_all()
+        return 1
 
     # this callback is for the entry signal at the event boxes, to update the tooltip value
     def update_info(self,widget,event):
-        info = self.read_proc_info();
-        self.tooltips.set_tip(self.box, info[0] + " " + info[1]);
+        self.read_proc_info()
 
     def change_orientation(self,arg1,data):
         self.orientation = self.applet.get_orient()
-        # first remove the children of the current box
-        for i in range(len(self.numbers)):
-            self.box.remove(self.numbers[i])
 
-        # now remove the box itself
-        self.big_evbox.remove(self.box)
-
-        # time to create the new box
         if self.orientation == gnome.applet.ORIENT_UP or self.orientation == gnome.applet.ORIENT_DOWN:
-            self.box = gtk.HBox()
+            tmpbox = gtk.HBox()
         else:
-            self.box = gtk.VBox()
+            tmpbox = gtk.VBox()
+        
+        # reparent all the hboxes to the new tmpbox
+        for i in (self.box.get_children()):
+            i.reparent(tmpbox)
 
-        # and now fill it with the numbers
-        for i in range(len(self.numbers)):
-            self.box.pack_start(self.numbers[i])
-
-        # final steps
+        # now remove the link between big_evbox and the box
+        self.big_evbox.remove(self.big_evbox.get_children()[0])
+        self.box = tmpbox
         self.big_evbox.add(self.box)
-        self.box.show()
+        self.applet.show_all()
 
+     # this callback is to create the context menu
+    def button_clicked(self,widget,event):
+        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
+            pass
+            #self.orientation = gnome.applet.ORIENT_LEFT
+            #self.change_orientation()
     # this callback is to create the context menu
     def button_press(self,widget,event):
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
             self.create_menu()
-            
+           
     
     # and this one to show the about box   
     def about_info(self,event,data=None):
-        about = gnome.ui.About("CPU performance stats applet","0.2","GPL","GNOME Applet to check throttling CPU states",["Arturo Gonzalez <arturogf@ugr.es>"],\
+        about = gnome.ui.About("CPU performance stats applet",pglobals.version,"GPL","GNOME Applet to check throttling CPU states",["Arturo Gonzalez <arturogf@ugr.es>"],\
                                ["Arturo Gonzalez <arturogf@ugr.es>"],"Arturo Gonzalez <arturogf@ugr.es>",self.logo_pixbuf)
         about.show()
 
@@ -89,62 +143,101 @@ class pycentrinoApplet(gnome.applet.Applet):
     # this function update the info extracted from /proc/acpi
     def read_proc_info(self):
 
+        # cpu power in mhz
+
+        if (os.path.isfile("/proc/cpuinfo")):
+            f_proc = open("/proc/cpuinfo")
+            for line in f_proc:
+                try:
+                    line.index("cpu MHz")
+                    self.info[5] = line.strip().replace(" ","").split(":")[1]
+                except ValueError:
+                    pass
+        
         # throttling support
         if (os.path.isfile("/proc/acpi/processor/CPU0/throttling")):
-            f_throt = open("/proc/acpi/processor/CPU0/throttling","r");
-            cont = 1;
+            f_throt = open("/proc/acpi/processor/CPU0/throttling","r")
+            cont = 1
 
             for line in f_throt:
                 if cont == 1:
                     tmp = re.split('\W+',line)
                     self.throttling_states = int(tmp[2])
                 else:
-                    if cont == 2:
-                        tmp = re.split('\W+', line)
-                        self.active_t_state = int(tmp[2][1])
-                    else:
+                    # search for * and print self.info
+                    try:
+                        line.index("*")
+                        tmp = line.strip().replace(" ","").replace("*","").split(":")
+                        self.info[0] = tmp[0]
+                        self.info[1] = str(100 - int(tmp[1][:-1])) + "%"
+                        print self.info[1]
+                        
+                        #print self.info
                         break
-                cont = cont + 1
+                    except ValueError:
+                        pass
                     
+                cont = cont + 1
             f_throt.close()
         else:
-            self.lack_t_acpi = 1
+            self.info[0] = ""
+            self.info[1] = ""
 
         # power support
         if (os.path.isfile("/proc/acpi/processor/CPU0/power")):
-            f_per = open("/proc/acpi/processor/CPU0/power","r");
-            cont = 1;
+            f_per = open("/proc/acpi/processor/CPU0/power","r")
+            cont = 1
 
             for line in f_per:
                 if cont == 1:
                     tmp = re.split('\W+',line)
                     self.active_cpu_state = tmp[2][1]
                 else:
-                    if cont == 3:
-                        tmp = re.split('\W+', line)
-                        self.bus_activity = tmp[3]
-                    else:
-                        if cont != 2:
-                            break
-
-                cont = cont + 1
+                    try:
+                        line.index("*")
+                        self.info[2] = line.strip().replace(" ","").replace("*","").split(":")[0]
+                        #print self.info
+                        break
+                    except ValueError:
+                        pass
                     
+                cont = cont + 1
             f_per.close()
         else:
-            self.lack_p_acpi = 1
+            self.info[2] = ""
 
-        # we return a tuple. In case there is no support, return a error string to be shown
-        if self.lack_t_acpi and self.lack_p_acpi:
-            return ("lack of throttling acpi support","lack of power acpi support")
+        # 2.6 sys filesystem
+        if (os.path.isfile("/sys/devices/system/cpu/cpu0/cpufreq/scaling_available_governors")):
+            f_sys = open("/sys/devices/system/cpu/cpu0/cpufreq/scaling_available_governors")
+            for line in f_sys:
+                self.governors = re.split(" ", line)
+            f_sys.close()
         else:
-            if self.lack_t_acpi:
-                return ("lack of throttling acpi support","")
-            else:
-                if self.lack_p_acpi:
-                   return("","lack of power acpi support")
-                else:
-                    return("","")
+            self.governors = ""
 
+            
+
+        if (os.path.isfile("/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor")):
+            f_sys = open("/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor")
+            for line in f_sys:
+                self.info[3] = line.strip()
+
+            f_sys.close()
+   
+
+        else:
+            self.info[3] = ""
+
+        if (os.path.isfile("/sys/devices/system/cpu/cpu0/cpufreq/scaling_driver")):
+            f_sys = open("/sys/devices/system/cpu/cpu0/cpufreq/scaling_driver")
+            for line in f_sys:
+                self.info[4] = line.strip()
+
+            f_sys.close()
+        else:
+            self.info[4] = ""
+
+        
 
     # this callback is to create a context menu with the second button
     def create_menu(self):
@@ -180,18 +273,13 @@ class pycentrinoApplet(gnome.applet.Applet):
         self.__gobject_init__()
 
         # instance variable section
+        self.info = ["T0","0%","C1","powersave","centrino","0.0"]
         self.timeout_interval = 1000        
         self.throttling_states = 0
-        self.active_t_state = "T0"
-        self.bus_activity = ""
-        self.actual_cpu_state = "C1"
-        self.lack_t_acpi = 0
-        self.lack_p_acpi = 0
-        self.prueba = {}
         self.tooltip_text = ""
-        self.checks = [0,0,0,0,0]
-        self.orientation = gnome.applet.ORIENT_UP
-        self.numbers = []
+        self.checks = [1,1,1,0,0,1]
+        self.orientation = gnome.applet.ORIENT_DOWN
+        self.logo_pixbuf = None
 
         self.propxml="""
         <popup name="button3">
@@ -203,51 +291,63 @@ class pycentrinoApplet(gnome.applet.Applet):
         self.verbs = [ ( "Props", self.properties ),
                        ( "About", self.about_info )  
 	             ]
-
+        self.logo_pixbuf 
         # initialize gnome application and set up all the gnome internals
         gnome.init(pglobals.name, pglobals.version)
 
         # create the logo from file centrino.gif
-        self.logo = gtk.Image()
-        self.logo.set_from_file(os.path.join(pglobals.image_dir,'centrino.gif'))
-        self.logo_pixbuf = self.logo.get_pixbuf()
-
-        # fill the numbers vector with images of numbers
-        for i in range(10):
-            self.numbers.append(gtk.Image())
-            self.numbers[i].set_from_file(os.path.join(pglobals.image_dir,'green_led_%d.gif' % i))
+        self.logo_pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(pglobals.image_dir,'centrino.gif'))
 
         # create the applet and the widgets
         self.applet = applet
-        self.tooltips = gtk.Tooltips()
+        #self.tooltips = gtk.Tooltips()
         self.big_evbox = gtk.EventBox()
         self.big_evbox.connect("button-press-event",self.button_press)
-        self.big_evbox.connect("enter-notify-event", self.update_info)
+        self.orientation = self.applet.get_orient()
+
         self.box = None
-
-        self.orientation = applet.get_orient()
-
-        # if we have an horizontal panel if self.orientation ==
+        # if we have an horizontal panel 
         if self.orientation == gnome.applet.ORIENT_UP or self.orientation == gnome.applet.ORIENT_DOWN:
             self.box = gtk.HBox()
         # if it's a vertical one
         else:
             self.box = gtk.VBox()
 
-        # now add the numbers to the box
-        for i in range(len(self.numbers)):
-            self.box.pack_start(self.numbers[i])
-
         self.big_evbox.add(self.box)
+        # TODO: read preferences from gconf
 
+        cont = -1
+        separate = 0
+        # create the applet with little hboxes containing each info checked in the preferences
+        for i in range(len(self.checks)):
+            if self.checks[i]:
+                cont = cont + 1
+
+                self.box.pack_start(gtk.EventBox())
+                
+                # add un hbox para meter imagenes
+                self.box.get_children()[cont].add(gtk.HBox())
+                if separate:
+                    tmp = gtk.Image()
+                    tmp.set_from_file(os.path.join(pglobals.image_dir,'separator.gif'))
+                    self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+
+                
+                for h in range(len(self.info[i])):
+                    tmp = gtk.Image()
+                    tmp.set_from_file(os.path.join(pglobals.image_dir,'green_led_%s.gif' % self.info[i][h]))
+                    self.box.get_children()[cont].get_children()[0].pack_start(tmp)
+                separate = 1
+                
         self.applet.add(self.big_evbox)        
 
         # update info from filesystem
         gtk.timeout_add(self.timeout_interval,self.timeout_callback, self)
         
         # connect some signals to the applet
+        self.applet.connect("button-press-event",self.button_clicked)
         self.applet.connect("change-orient",self.change_orientation)
-        self.applet.connect("destroy",self.cleanup)
+        self.applet.connect("delete-event",self.cleanup)
         self.applet.show_all()
 
         # read the .glade file to construct the properties window
